/*
**	CursorPanel.c
**
**	Editing panel for cursor key configuration
**
**	Copyright  1990-1993 by Olaf `Olsen' Barthel & MXM
**		All Rights Reserved
*/

#include "termGlobal.h"

enum	{	GAD_UP,GAD_DOWN,GAD_RIGHT,GAD_LEFT,GAD_QUALIFIER,
		GAD_USE,GAD_LOAD,GAD_SAVE,GAD_CANCEL };

STATIC STRPTR ModifierString[5];

STATIC struct Gadget *
CreateAllGadgets(struct Gadget **GadgetArray,struct Gadget **GadgetList)
{
	struct Gadget		*Gadget;
	struct NewGadget	 NewGadget;
	UWORD			 Counter = 0;

	LocalizeString(ModifierString,MSG_MACROPANEL_NONE_TXT,MSG_MACROPANEL_CONTROL_TXT);

	SZ_SizeSetup(Window -> WScreen,&UserFont,TRUE);

	memset(&NewGadget,0,sizeof(struct NewGadget));

	if(Gadget = CreateContext(GadgetList))
	{
		SZ_AddLeftOffset(SZ_LeftOffsetDelta(MSG_CURSORPANEL_CURSOR_UP_GAD,MSG_CURSORPANEL_CURSOR_RIGHT_GAD));

		SZ_SetWidth(SZ_Width(STRING_KIND,NULL,40,NULL));

		NewGadget . ng_GadgetText	= LocaleString(MSG_CURSORPANEL_CURSOR_UP_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(STRING_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,

			GTST_MaxChars,	255,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_CURSORPANEL_CURSOR_DOWN_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(STRING_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,

			GTST_MaxChars,	255,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_CURSORPANEL_CURSOR_RIGHT_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(STRING_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,

			GTST_MaxChars,	255,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_CURSORPANEL_CURSOR_LEFT_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(STRING_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,

			GTST_MaxChars,	255,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_MACROPANEL_KEY_MODIFIER_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(CYCLE_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AlignCentre,	TRUE,

			GTCY_Labels,	ModifierString,
			GTCY_Active,	0,
		TAG_DONE);

		SZ_UpdateMaxWidth(BUTTON_KIND,LocaleString(MSG_GLOBAL_USE_GAD),0,NULL);
		SZ_UpdateMaxWidth(BUTTON_KIND,LocaleString(MSG_GLOBAL_CANCEL_GAD),0,NULL);
		SZ_UpdateMaxWidth(BUTTON_KIND,LocaleString(MSG_GLOBAL_LOAD_GAD),0,NULL);
		SZ_UpdateMaxWidth(BUTTON_KIND,LocaleString(MSG_GLOBAL_CANCEL_GAD),0,NULL);

		SZ_SetWidth(SZ_ResetMaxWidth());

		NewGadget . ng_GadgetText	= LocaleString(MSG_GLOBAL_USE_GAD);
		NewGadget . ng_GadgetID		= Counter;
		NewGadget . ng_Flags		= 0;

		GadgetArray[Counter++] = Gadget = CreateGadget(BUTTON_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,
			SZ_AlignExtra,	TRUE,
			SZ_AlignBottom,	TRUE,
			SZ_GroupCount,	4,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_GLOBAL_LOAD_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(BUTTON_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,
			SZ_GroupNext,	TRUE,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_GLOBAL_SAVE_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(BUTTON_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,
			SZ_GroupNext,	TRUE,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_GLOBAL_CANCEL_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter] = Gadget = CreateGadget(BUTTON_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,
			SZ_GroupNext,	TRUE,
		TAG_DONE);
	}

	return(Gadget);
}

VOID
CursorPanel(struct CursorKeys *CursorKeys)
{
	struct Window		*PanelWindow;
	struct Gadget		*GadgetList = NULL;
	struct Gadget		*GadgetArray[GAD_CANCEL + 1];
	LONG			 i;

	struct CursorKeys	*Keys;
	ULONG			 OldCRC;

	OldCRC = DoCRC(CursorKeys,sizeof(struct CursorKeys));

	if(Keys = (struct CursorKeys *)AllocVec(sizeof(struct CursorKeys),MEMF_ANY|MEMF_CLEAR))
	{
		CopyMem(CursorKeys,Keys,sizeof(struct CursorKeys));

		if(CreateAllGadgets(GadgetArray,&GadgetList))
		{
			if(PanelWindow = OpenWindowTags(NULL,
				WA_Left,	GetScreenLeft(Window) + (GetScreenWidth(Window) - SZ_GetWindowWidth()) / 2,
				WA_Top,		GetScreenTop(Window) + (GetScreenHeight(Window) - SZ_GetWindowHeight())	/ 2,
				WA_Width,	SZ_GetWindowWidth(),
				WA_Height,	SZ_GetWindowHeight(),

				WA_Activate,	TRUE,
				WA_DragBar,	TRUE,
				WA_DepthGadget,	TRUE,
				WA_CloseGadget,	TRUE,
				WA_RMBTrap,	TRUE,
				WA_DepthGadget,	TRUE,
				WA_CustomScreen,Window -> WScreen,

				WA_IDCMP,	IDCMP_GADGETDOWN | IDCMP_CLOSEWINDOW | IDCMP_RAWKEY | IDCMP_VANILLAKEY | IDCMP_ACTIVEWINDOW | BUTTONIDCMP | CYCLEIDCMP | STRINGIDCMP,

				WA_Title,	LocaleString(MSG_CURSORPANEL_CURSOR_PREFERENCES_TXT),
			TAG_DONE))
			{
				struct IntuiMessage	*Massage;
				ULONG			 IClass,Code;
				struct Gadget		*Gadget;
				BYTE			 Terminated = FALSE;

				UBYTE			 DummyBuffer[MAX_FILENAME_LENGTH],
							*DummyChar;

				struct FileRequester	*FileRequest;

				LONG			 Modifier = 0;

				GuideContext(CONTEXT_CURSOR);

				PushWindow(PanelWindow);

				AddGList(PanelWindow,GadgetList,(UWORD)-1,(UWORD)-1,NULL);
				RefreshGList(GadgetList,PanelWindow,NULL,(UWORD)-1);
				GT_RefreshWindow(PanelWindow,NULL);

				for(i = 0 ; i < 4 ; i++)
				{
					GT_SetGadgetAttrs(GadgetArray[i],PanelWindow,NULL,
						GTST_String,Keys -> Keys[Modifier][i],
					TAG_DONE);
				}

				ActiveGadget = NULL;

				SetSignal(0,SIG_BREAK);

				while(!Terminated)
				{
					if(Wait(PORTMASK(PanelWindow -> UserPort) | SIG_BREAK) & SIG_BREAK)
					{
						CopyMem(Keys,CursorKeys,sizeof(struct CursorKeys));

						break;
					}

					while(!Terminated && (Massage = (struct IntuiMessage *)GT_GetIMsg(PanelWindow -> UserPort)))
					{
						IClass	= Massage -> Class;
						Code	= Massage -> Code;
						Gadget	= (struct Gadget *)Massage -> IAddress;

						GT_ReplyIMsg(Massage);

						if(IClass == IDCMP_ACTIVEWINDOW && ActiveGadget)
							ActivateGadget(ActiveGadget,PanelWindow,NULL);

						KeySelect(GadgetArray,GAD_CANCEL,Code,PanelWindow,&Gadget,&IClass,&Code);

						if(IClass == IDCMP_GADGETDOWN)
						{
							if((Gadget -> GadgetType & GTYP_GTYPEMASK) == GTYP_STRGADGET)
								ActiveGadget = Gadget;
						}

						if(IClass == IDCMP_CLOSEWINDOW)
						{
							CopyMem(Keys,CursorKeys,sizeof(struct CursorKeys));

							Terminated = TRUE;
						}

						if(IClass == IDCMP_GADGETUP)
						{
							switch(Gadget -> GadgetID)
							{
								case GAD_QUALIFIER:

									for(i = 0 ; i < 4 ; i++)
										strcpy(Keys -> Keys[Modifier][i],SZ_GetString(GadgetArray[i]));

									Modifier = Code;

									for(i = 0 ; i < 4 ; i++)
									{
										GT_SetGadgetAttrs(GadgetArray[i],PanelWindow,NULL,
											GTST_String,Keys -> Keys[Modifier][i],
										TAG_DONE);
									}

									break;

								case GAD_USE:

									Terminated = TRUE;

									for(i = 0 ; i < 4 ; i++)
										strcpy(Keys -> Keys[Modifier][i],SZ_GetString(GadgetArray[i]));

									CopyMem(Keys,CursorKeys,sizeof(struct CursorKeys));

									break;

								case GAD_LOAD:

									strcpy(DummyBuffer,LastCursorKeys);

									DummyChar = PathPart(DummyBuffer);

									*DummyChar = 0;

									BlockWindow(PanelWindow);

									if(FileRequest = GetFile(LocaleString(MSG_CURSORPANEL_LOAD_CURSOR_KEYS_TXT),DummyBuffer,FilePart(LastCursorKeys),DummyBuffer,"#?.prefs",FALSE,FALSE,FALSE,LocaleString(MSG_GLOBAL_LOAD_TXT)))
									{
										if(!ReadIFFData(DummyBuffer,Keys,sizeof(struct CursorKeys),ID_KEYS))
											MyEasyRequest(Window,LocaleString(MSG_CURSORPANEL_COULD_NOT_LOAD_CURSOR_KEYS_TXT),LocaleString(MSG_GLOBAL_CONTINUE_TXT),DummyBuffer);
										else
										{
											OldCRC = DoCRC(Keys,sizeof(struct CursorKeys));

											CursorKeysChanged = FALSE;

											strcpy(Config -> FileConfig -> CursorFileName,DummyBuffer);

											strcpy(LastCursorKeys,DummyBuffer);

											for(i = 0 ; i < 4 ; i++)
											{
												GT_SetGadgetAttrs(GadgetArray[i],PanelWindow,NULL,
													GTST_String,Keys -> Keys[Modifier][i],
												TAG_DONE);
											}
										}
									}

									ReleaseWindow(PanelWindow);

									break;

								case GAD_SAVE:

									strcpy(DummyBuffer,LastCursorKeys);

									DummyChar = PathPart(DummyBuffer);

									*DummyChar = 0;

									BlockWindow(PanelWindow);

									for(i = 0 ; i < 4 ; i++)
										strcpy(Keys -> Keys[Modifier][i],SZ_GetString(GadgetArray[i]));

									if(FileRequest = GetFile(LocaleString(MSG_CURSORPANEL_SAVE_CURSOR_KEYS_TXT),DummyBuffer,FilePart(LastCursorKeys),DummyBuffer,"#?.prefs",TRUE,FALSE,FALSE,LocaleString(MSG_GLOBAL_SAVE_TXT)))
									{
										if(!WriteIFFData(DummyBuffer,Keys,sizeof(struct CursorKeys),ID_KEYS))
											MyEasyRequest(Window,LocaleString(MSG_CURSORPANEL_COULD_NOT_SAVE_CURSOR_KEYS_TXT),LocaleString(MSG_GLOBAL_CONTINUE_TXT),DummyBuffer);
										else
										{
											strcpy(LastCursorKeys,DummyBuffer);

											CursorKeysChanged = FALSE;

											OldCRC = DoCRC(Keys,sizeof(struct CursorKeys));
										}
									}

									ReleaseWindow(PanelWindow);

									break;

								case GAD_CANCEL:

									Terminated = TRUE;

									CopyMem(Keys,CursorKeys,sizeof(struct CursorKeys));

									break;
							}
						}
					}
				}

				RemoveGList(PanelWindow,GadgetList,(UWORD)-1);

				PopWindow();

				CloseWindow(PanelWindow);
			}
		}

		FreeGadgets(GadgetList);

		FreeVec(Keys);
	}

	CursorKeysChanged |= (OldCRC != DoCRC(CursorKeys,sizeof(struct CursorKeys)));
}
